# -*- coding: utf-8 -*-
from setuphelpers import *
import platform
import json

uninstallkey = []

# Declaring specific app values (TO CHANGE)
bin_name_deb_string = 'code-%s_amd64.deb'
bin_name_rpm_string = 'code-%s.x86_64.rpm'
package_name = 'code'
app_dir = makepath('/','usr','share','code')
ext_python = 'ms-python-release.vsix'
ext_path = makepath(app_dir,ext_python)
deb_source = makepath('/','etc','apt','sources.list.d','vscode.list')
yum_source = makepath('/','etc','yum.repos.d','vscode.repo')


def install():

    isapt = True
    try:
        run('apt-get --version')
    except:
        isapt = False

    # Declaring specific app values
    package_version = control.version.split('-')[0]
    bin_name_deb = bin_name_deb_string % package_version
    bin_name_rpm = bin_name_rpm_string  % package_version

    # Installing the package

    if isapt:
        print('Installing: %s' % bin_name_deb)
        install_deb(bin_name_deb)
    else:
        print('Installing: %s' % bin_name_rpm)
        install_yum(bin_name_rpm)

    # Deleting sources files
    if isfile(deb_source):
        print('Removing: ' + deb_source)
        remove_file(deb_source)
    if isfile(yum_source):
        print('Removing: ' + yum_source)
        remove_file(yum_source)

    # Copying extension files to vscode dir
    print('Copying %s to %s' % (ext_python,app_dir))
    filecopyto(ext_python,app_dir)


def uninstall():
    # Uninstalling the package
    if platform.linux_distribution()[0].lower() in ['debian','ubuntu']:
        print('Uninstalling: %s' % package_name)
        uninstall_apt(package_name)

    if platform.linux_distribution()[0].lower() in ['centos linux','centos','suse','fedora','redhat']:
        print('Uninstalling: %s' % package_name)
        uninstall_yum(package_name)


def session_setup():
    print('Disabling : telemetry, check for update, auto-update, etc.')

    #https://developer.hyvor.com/vscode-editing-settings-json

    #/root/.config/Code/User/settings.json
    #/home/user/.config/Code/User/settings.json

    settings_json_content = {'telemetry.enableTelemetry':False,'telemetry.enableCrashReporter':False,'update.mode':"none","update.enableWindowsBackgroundUpdates":False,"update.showReleaseNotes":False,"extensions.autoUpdate":False,"extensions.autoCheckUpdates":False,"editor.minimap.enabled":False,"editor.wordWrap":"on","remote.SSH.allowLocalServerDownload":False}
    conf_dir_path = makepath(user_home_directory(),'.config','Code','User')
    conf_file_path = makepath(conf_dir_path,'settings.json')

    if not isdir(conf_dir_path):
        mkdirs(conf_dir_path)
    if not isfile(conf_file_path):
        with open(conf_file_path,'w+') as outfile:
            json.dump(settings_json_content,outfile,indent=4)
    else:
        with open(conf_file_path,'r+') as f:
            settings_json_outfile = json.load(f)
            settings_json_outfile.update(settings_json_content)
            f.seek(0, 0)
            json.dump(settings_json_outfile,f,indent=4)

    print('Installing Python extension')
    run('code --install-extension "%s"' % ext_path)


def update_package():
    print('Download/Update package content from upstream binary sources')

    # Getting proxy informations from WAPT settings
    proxy = {}
    if platform.system()=='Windows' and isfile(makepath(user_local_appdata(),'waptconsole','waptconsole.ini')):
        proxywapt = inifile_readstring(makepath(user_local_appdata(),'waptconsole','waptconsole.ini'),'global','http_proxy')
        if proxywapt :
            proxy = {'http':proxywapt,'https':proxywapt}

    # Specific app values
    app_name = control.name
    url_api = 'https://update.code.visualstudio.com/api/update/linux-x64/stable/VERSION'

    # Getting latest informations from Github API
    json_load = json.loads(wgets(url_api,proxies=proxy))
    version = (json_load['name'])

    #url_dl_deb = 'https://go.microsoft.com/fwlink/?LinkID=760868'
    #url_dl_yum = 'https://go.microsoft.com/fwlink/?LinkID=760867'
    url_dl_deb = 'https://update.code.visualstudio.com/%s/linux-deb-x64/stable' % version
    url_dl_yum = 'https://update.code.visualstudio.com/%s/linux-rpm-x64/stable' % version

    latest_bin_deb = bin_name_deb_string % version
    latest_bin_yum = bin_name_rpm_string % version

    print('Latest ' + app_name + ' version is: ' + version)
    print('Download url for APT-based systems is: ' + url_dl_deb)
    print('Download url for YUM-based systems is: ' + url_dl_yum)

    # Downloading latest binaries
    if not isfile(latest_bin_deb):
        print('Downloading: ' + latest_bin_deb)
        wget(url_dl_deb,latest_bin_deb,proxies=proxy)

    if not isfile(latest_bin_yum):
        print('Downloading: ' + latest_bin_yum)
        wget(url_dl_yum,latest_bin_yum,proxies=proxy)

		# Change version of the package
        pe = PackageEntry().load_control_from_wapt('.')
        pe.version = '%s-%s'%(version,int(pe.version.split('-',1)[1])+1)
        pe.save_control_to_wapt('.')
        print('Changing version to ' + pe.version + ' in WAPT\\control')
        print('Update package done. You can now build-upload your package')
    else:
        print('This package is already up-to-date')

    # Downloading latest extensions
    json_load = json.loads(wgets('https://api.github.com/repos/microsoft/vscode-python/releases/latest',proxies=proxy))
    for download_ext in json_load['assets']:
        if download_ext['browser_download_url'].endswith('.vsix'):
            url_dl_ext = download_ext['browser_download_url']
            if isfile(ext_python):
                remove_file(ext_python)
            print('Downloading: ' + ext_python)
            wget(url_dl_ext,ext_python,proxies=proxy)
            break

    # Deleting outdated binaries
    for bin_in_dir in glob.glob('*.deb'):
        if bin_in_dir != latest_bin_deb:
            print('Outdated binary: ' + bin_in_dir + ' Deleted')
            remove_file(bin_in_dir)

    # Deleting outdated binaries
    for bin_in_dir in glob.glob('*.rpm'):
        if bin_in_dir != latest_bin_yum :
            print('Outdated binary: ' + bin_in_dir + ' Deleted')
            remove_file(bin_in_dir)

